# Examples 6.3 and 6.5 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################


### Example 6.3.
data(afterthin)
library(lmfor)

windows(width=7,height=3)
par(mfcol=c(1,2),cex=0.9,mai=c(0.6,0.6,0.5,0.1), mgp=c(2,0.7,0))
linesplot(afterthin$CA,
          afterthin$RBA,
          group=afterthin$Plot:afterthin$Tree,
          col.lin=gray(1-(as.numeric(afterthin$SDClass)+1)/6),
          cex=0,
          lwd=as.numeric(afterthin$SDClass)/2+0.5,
          xlab="Tree age",
          ylab=expression("Ring basal area, "*mm^2),
          main="All Data")
legend("topleft",
       legend=c("Control","Light","Moderate","Heavy"),
       lwd=as.numeric(c(1,2,3,4))/2+0.5,
       col=unique(gray(1-as.numeric(afterthin$SDClass)/5))[c(1,4,3,2)],
       cex=0.9)
linesplot(afterthin$CA[afterthin$SDClass==2],
          afterthin$RBA[afterthin$SDClass==2],
          group=(afterthin$Plot:afterthin$Tree)[afterthin$SDClass==2],
          cex=0,
          col.lin=gray((as.numeric(afterthin$Plot)[afterthin$SDClass==2]-4)/8),
          xlab="Tree age",
          ylab=expression("Ring basal area, "*mm^2),
          main="SDClass=2 (3 plots)",
          lwd=2)

#aftmod1<-lme(RBA~CA+SDClass,
#	 	     random=~1|Plot/Tree,
#		     data=afterthin)
  
#summary(aftmod1)
#pdf("d:/laurim/biometria/lecturenotes/figpat2.pdf")
#plot(predict(aftmod1),resid(aftmod1))
#mywhiskers(predict(aftmod1),resid(aftmod1),add=TRUE,se=FALSE)
#dev.off()

aftmod2<-lme(RBA~CA+SDClass,
             random=~1|Plot/Tree,
             data=afterthin,
             weights=varPower())
  
windows(width=7,height=3)
par(mfcol=c(1,2),cex=0.9,mai=c(0.6,0.6,0.1,0.1), mgp=c(2,0.7,0))

plot(predict(aftmod2),resid(aftmod2),ylab="Raw residual",xlab="Fitted value",cex=0.1)
mywhiskers(predict(aftmod2),resid(aftmod2),add=TRUE,se=FALSE)
abline(0,0)

plot(predict(aftmod2),resid(aftmod2,type="p"),
     ylab="Standardized residual",xlab="Fitted value",cex=0.1)
mywhiskers(predict(aftmod2),resid(aftmod2,type="p"),add=TRUE,se=FALSE)
abline(0,0)


anova(aftmod2,type="m")
summary(aftmod2)

ranef(aftmod2,level=1)[1:5,]
ranef(aftmod2,level=2)[1:5,]

fixef(aftmod2)
coef(aftmod2,level=1)
coef(aftmod2,level=2)[1:5,]

aftmod2NoReNoVF<-gls(RBA~CA+SDClass,data=afterthin)
aftmod2NoReVF<-gls(RBA~CA+SDClass,data=afterthin,weights=varPower())
aftmod2PlotOnlyVF<-lme(RBA~CA+SDClass,random=~1|Plot,
                       data=afterthin,weights=varPower())
anova(aftmod2NoReNoVF,type="m")
anova(aftmod2NoReVF,type="m")
anova(aftmod2PlotOnlyVF,type="m")


###

### Example 6.5

library(lme4)

windows(width=7,height=3)
par(mfcol=c(1,2),cex=0.9,mai=c(0.6,0.6,0.5,0.1), mgp=c(2,0.7,0))
linesplot(afterthin$Year,
		afterthin$RBA,
		group=afterthin$Plot:afterthin$Tree,
		col.lin=gray(1-(as.numeric(afterthin$SDClass)+1)/6),
		lwd=as.numeric(afterthin$SDClass)/2+0.5,
		cex=0,
		xlab="Calendar Year",
		ylab=expression("Ring basal area, "*mm^2),
		main="All Data")
legend("topleft",
		legend=c("Control","Light","Moderate","Heavy"),
		lwd=as.numeric(c(1,2,3,4))/2+0.5,
		col=unique(gray(1-as.numeric(afterthin$SDClass)/5))[c(1,4,3,2)],
		cex=0.75)
linesplot(afterthin$Year[afterthin$SDClass==2],
		afterthin$RBA[afterthin$SDClass==2],
		group=(afterthin$Plot:afterthin$Tree)[afterthin$SDClass==2],
		cex=0,
		col.lin=gray((as.numeric(afterthin$Plot)[afterthin$SDClass==2]-4)/8),
		xlab="Calendar Year",
		ylab=expression("Ring basal area, "*mm^2),
		main="SDClass=2 (3 plots)",
		lwd=2)


# Recode tree so that numbering does not start from 1 for each plot. 
# Each tree will now get a unique id in variable "Tree2".
afterthin$Tree2<-with(afterthin,factor(Plot:Tree))

aftmod3<-lmer(RBA~SDClass+CA+(1|Plot)+(1|Tree2)+(1|Year),data=afterthin)  
# We could use RBA~SDClass+CA+(1|Plot/Tree)+(1|Year) as well.
for (i in 1:20){
	cat(i,logLik(aftmod3),"\n")
	afterthin$ytilde<-fitted(aftmod3)
	aftmod3<-lmer(RBA~SDClass+CA+(1|Plot)+(1|Tree2)+(1|Year),
			weights=1/ytilde^{1.6},
			data=afterthin)  
}

windows(width=7,height=3)
par(mfcol=c(1,2),cex=0.9,mai=c(0.6,0.6,0.1,0.1), mgp=c(2,0.7,0))

plot(fitted(aftmod3),resid(aftmod3),cex=0.1,xlab="Fitted value",ylab="Raw residual")
mywhiskers(fitted(aftmod3),resid(aftmod3),add=TRUE,se=FALSE)

plot(fitted(aftmod3),resid(aftmod3,type="pearson"),cex=0.1,xlab="Fitted value",ylab="Pearson residual")
mywhiskers(fitted(aftmod3),resid(aftmod3,type="pearson"),add=TRUE,se=FALSE)


summary(aftmod3)

aftmod4<-update(aftmod3,formula=RBA~SDClass+CA+(1|Tree2)+(1|Year))
for (i in 1:20){
	cat(i,logLik(aftmod4),"\n")
	afterthin$ytilde<-fitted(aftmod4)
	aftmod4<-lmer(RBA~SDClass+CA+(1|Tree2)+(1|Year),
			weights=1/ytilde^{1.6},
			data=afterthin)  
}

anova(aftmod3,aftmod4)


library(lmerTest) 
aftmod4<-update(aftmod4)
anova(aftmod4)

summary(aftmod4)

# This will take some time
confint(aftmod4,method="boot")

windows(7,3)
par(mfcol=c(1,2),cex=0.9,mai=c(0.6,0.6,0.5,0.1), mgp=c(2,0.7,0))
barplot(ranef(aftmod4)$Year[,1],
        names=rownames(ranef(aftmod4)$Year),
        main="Year effects",xlab="Calendar year",
        ylab=expression("Year effect "*mm^2/yr))
barplot(ranef(aftmod4)$Tree[,1],
        names=rownames(ranef(aftmod4)$Tree),
        main="Tree effects",xlab="Plot id:Tree id",
        ylab=expression("Tree effect "*mm^2/yr))


predTree<-predict(aftmod4,newdata=afterthin,re.form=~(1|Tree2))
predPop<-predict(aftmod4,newdata=afterthin,re.form=~0)
predYear<-predict(aftmod4,newdata=afterthin,re.form=~(1|Year))
predAll<-predict(aftmod4,newdata=afterthin,re.form=~(1|Tree2)+(1|Year))



# Crossed effects using package nlme
# Not included in the book.
detach(package:lme4)
library(nlme)
afterthin$Year<-as.factor(afterthin$Year)
afterthin<-groupedData(RBA~SDClass|Plot,data=afterthin)
mod3cNLME<-lme(RBA~SDClass+CA, 
		random=pdBlocked(list(pdIdent(~Tree2-1), pdIdent(~Year-1))),
		data=afterthin)

mod4cNLME<-lme(RBA~SDClass+CA, 
		random=pdBlocked(list(pdIdent(~Tree2-1), pdIdent(~Year-1))),
		data=afterthin,
		weights=varPower())

summary(mod4cNLME)



